CustomMarker.prototype = new google.maps.OverlayView();
function CustomMarker(options) {
    // Initialize all properties.
    this.marker = options;
    this.icon = options.icon;
    this.title = options.title;

    // Define a property to hold the image's div. We'll
    // actually create this div upon receipt of the onAdd()
    // method so we'll leave it null for now.
    this.div_ = null;
    this.img_ = null;
    this.haveDoneOffset = false;
    this.drawOffset = { x: 0, y: 0 };

    // Explicitly call setMap on this overlay.
    this.setMap(this.marker.map);
}

/**
* onAdd is called when the map's panes are ready and the overlay has been
* added to the map.
*/
CustomMarker.prototype.onAdd = function() {
    var div = document.createElement('div');
    div.style.position = 'absolute';

    // Create the img element and attach it to the div.
    var img = document.createElement('img');
    img.src = this.marker.icon.url;
    img.style.width = '100%';
    img.style.height = '100%';
    img.style.position = 'absolute';
    img.style.cursor = 'pointer';
    img.className = 'Thumbnail';
    div.appendChild(img);

    var canvas = document.createElement('canvas');
    canvas.style.position = 'absolute';
    /*
    canvas.style.backgroundColor = 'blue';
    canvas.style.opacity = 0.5;
    */
    div.appendChild(canvas);

    this.div_ = div;
    this.img_ = img;
    this.canvas_ = canvas;

    // Add the element to the "overlayLayer" pane.
    var panes = this.getPanes();
    panes.overlayMouseTarget.appendChild(div);

    var that = this;
    google.maps.event.addDomListener(img, 'click', function(event) {
        // The infoBubble gets it's position from the marker's position.
        google.maps.event.trigger(that, 'click', {
            stop: null,
            latLng: that.marker.position
        });
    });
};

CustomMarker.prototype.getDraggable = function() {
    return false;
};

CustomMarker.prototype.setVisible = function(show) {
    if (this.div_) {
        this.div_.style.display = (show ? "" : "none");
    }
};

CustomMarker.prototype.getPosition = function() {
    return this.marker.position;
};

CustomMarker.prototype.getRectForZoom = function(zoom) {
    var overlayProjection = this.getProjection();
    var markerPos = overlayProjection.fromLatLngToDivPixel(this.marker.position);
    var displayWidth = (this.img_.naturalWidth * 0.5 * Math.max(1, zoom - 15));
    var displayHeight = (this.img_.naturalHeight * 0.5 * Math.max(1, zoom - 15));
    var res = {
        // x,y = centerpoint.
        x: markerPos.x,
        y: markerPos.y,
        // w/h == image width/height.
        w: displayWidth,
        h: displayHeight,
        // dx/dy = display top/left.
        dl: markerPos.x - (this.img_.clientLeft + (displayWidth / 2)),
        dt: markerPos.y - (this.img_.clientTop + (displayHeight / 2)),
        dr: markerPos.x + (this.img_.clientLeft + (displayWidth / 2)),
        db: markerPos.y + (this.img_.clientTop + (displayHeight / 2))
    };

    return res;
};

CustomMarker.prototype.draw = function() {
    // Resize the image's div to fit the indicated dimensions.
    var div = this.div_;
    var offsetX = 0;
    var offsetY = 0;
    var divPos = { x: 0, y: 0 };
    var rect = this.getRectForZoom(map.zoom);

    if (this.img_ != null) {
        // DIV top/left is the correct point. Center the image on it.
        offsetX = this.img_.clientLeft + (rect.w / 2);
        offsetY = this.img_.clientTop + (rect.h / 2);

        divPos.x = this.drawOffset.x + (rect.x - offsetX);
        divPos.y = this.drawOffset.y + (rect.y - offsetY);

        div.style.left = divPos.x + 'px';
        div.style.top = divPos.y + 'px';
        div.style.width = rect.w + 'px';
        div.style.height = rect.h + 'px';
    }

    if (this.canvas_ != null) {
        var context = this.canvas_.getContext("2d");

        if (context) {
            var pad = 5;

            context.clearRect(0, 0, this.canvas_.width, this.canvas_.height);
            this.canvas_.style.zIndex = this.img_.style.zIndex - 1;

            if (this.drawOffset.x != 0 || this.drawOffset.y != 0) {
                var drawToPoint = {
                    x: rect.x + this.drawOffset.x,
                    y: rect.y + this.drawOffset.y
                };

                var canvasPos = {
                    x: Math.min(drawToPoint.x, rect.x),
                    y: Math.min(drawToPoint.y, rect.y)
                };

                this.canvas_.style.left = ((canvasPos.x - divPos.x) - pad) + "px";
                this.canvas_.style.top = ((canvasPos.y - divPos.y) - pad) + "px";
                this.canvas_.width = (pad * 2) + Math.abs(drawToPoint.x - rect.x);
                this.canvas_.height = (pad * 2) + Math.abs(drawToPoint.y - rect.y);

                context.beginPath();
                context.moveTo(pad + (rect.x - canvasPos.x), pad + (rect.y - canvasPos.y));
                context.lineTo(pad + (drawToPoint.x - canvasPos.x), pad + (drawToPoint.y - canvasPos.y));
                context.lineWidth = 2;
                context.strokeStyle = "#222222";
                context.lineCap = "round";
                context.stroke();
            }
        }
    }
};

// The onRemove() method will be called automatically from the API if
// we ever set the overlay's map property to 'null'.
CustomMarker.prototype.onRemove = function() {
    this.div_.parentNode.removeChild(this.div_);
    this.div_ = null;
    this.img_ = null;
    this.canvas_ = null;
};
